import asyncio
import base64
import logging
from typing import Callable

import ddddocr
from httpx import AsyncClient, ReadTimeout
from playwright.async_api import async_playwright, TimeoutError

from ey_commerce_lib.four_seller.constant.response import ResponseCodeEnum
from ey_commerce_lib.four_seller.parser.order import parse_order
from ey_commerce_lib.four_seller.schemas.query.order import FourSellerOrderQueryModel
from ey_commerce_lib.four_seller.schemas.vo.order import FourSellerOrderVO
from ey_commerce_lib.model import Page
from ey_commerce_lib.utils.close import safe_close

logger = logging.getLogger(__name__)


class FourSellerClient:

    def __init__(self, user_name: str,
                 password: str,
                 user_token: str,
                 login_success_call_back: Callable,
                 sem: int = 10):
        self.user_name = user_name
        self.password = password
        self.user_token = user_token
        # 登录成功回调函数
        self.login_success_call_back = login_success_call_back
        # 信号量
        self.__sem = asyncio.Semaphore(sem)
        # 自动登录锁
        self.__login_lock = asyncio.Lock()
        # 当前登录任务的 future
        self.__login_in_progress = None
        # ddddocr实例
        self.__ocr = ddddocr.DdddOcr()
        # 异步客户端
        self.__async_client = AsyncClient(base_url="https://www.4seller.com", cookies={"userToken": user_token},
                                          headers={
                                              "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                                                            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/137.0.0.0 "
                                                            "Safari/537.36"
                                          })

    async def __auto_login_4seller(self) -> str:
        """
        自动登录4seller
        :return: 登录
        """
        browser = None
        context = None
        page = None
        try:
            async with async_playwright() as p:
                browser = await p.chromium.launch(
                    headless=True,
                    args=[
                        '--start-maximized',
                        '--disable-blink-features=AutomationControlled',
                    ]
                )

                context = await browser.new_context(
                    user_agent="Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
                               "(KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
                    viewport={"width": 1920, "height": 1080},
                    locale='zh-CN',  # 中文环境
                    timezone_id='Asia/Shanghai',  # 中国时区
                    geolocation={"longitude": 116.4074, "latitude": 39.9042},  # 北京位置
                    permissions=["geolocation"]
                )
                page = await context.new_page()
                await page.goto('https://www.4seller.com/login.html')
                # 等待元素出现
                await page.locator('.el-input__inner').first.wait_for()
                input_locators = await page.locator('.el-input__inner').all()
                username_input_element = input_locators[0]
                password_input_element = input_locators[1]
                await username_input_element.fill(self.user_name)
                await password_input_element.fill(self.password)
                login_button_element = page.locator('.el-button.sign_up')
                await login_button_element.click()
                # 获取验证码图片
                capcha_code_img_element = page.locator('.el-input__wrapper img')
                await capcha_code_img_element.wait_for()
                captcha_code_input_element = page.locator('[autocomplete="new-password"]')
                login_count = 1
                # 只能允许一百次登录
                while login_count <= 100:
                    # 等图片 src 被赋值
                    max_attempts = 10
                    for attempt in range(max_attempts):
                        src = await capcha_code_img_element.get_attribute("src")
                        if src and src.strip() != "":
                            break
                        await page.wait_for_timeout(500)
                    else:
                        raise Exception("获取不到图片验证码")
                    # 识别验证码
                    # 如果有前缀 "data:image/xxx;base64,", 需要去掉
                    if ',' in src:
                        base64_str = src.split(',')[1]

                    # 解码 base64 为字节流
                    img_bytes = base64.b64decode(base64_str)
                    captcha_code = self.__ocr.classification(img_bytes)
                    # 回填验证码
                    await captcha_code_input_element.fill(captcha_code)
                    # 再次点击登录按钮
                    await login_button_element.click()
                    try:
                        await page.wait_for_selector(".el-message.el-message--error", timeout=10000)
                    except TimeoutError:
                        # 获取userToken的cookie
                        cookies = await context.cookies()
                        for cookie in cookies:
                            if cookie["name"] == "userToken":
                                userToken = cookie["value"]
                                # 返回token
                                return userToken
                    # 防止验证码切换过快
                    await page.wait_for_timeout(2000)
                    login_count += 1
        except Exception as e:
            raise e
        finally:
            # 不管成功与否函数都要关闭
            for close_obj in (page, context, browser):
                await safe_close(close_obj)

    async def __refresh_user_token(self):
        """
        刷新token
        :return:
        """
        # 如果有登录任务正在运行
        if self.__login_in_progress is not None:
            # 已有协程在处理登录，我们等待它完成
            await self.__login_in_progress
            return

        # 没有登录任务，当前协程负责登录
        self.__login_in_progress = asyncio.get_event_loop().create_future()
        # 检测锁，确保只有一个线程在登录
        async with self.__login_lock:
            try:
                user_token = await self.__auto_login_4seller()
            except Exception as e:
                # 如果future没有被终结掉手动设置终结future
                if not self.__login_in_progress.done():
                    self.__login_in_progress.set_exception(e)
                logger.error(f'FourSeller自动登录失败 {e}')
                raise Exception(f'FourSeller登录身份失效, 尝试自动登录失败 {e}')
            else:
                # 登录成功后重置token
                self.user_token = user_token
                # 清空原cookies
                self.__async_client.cookies.clear()
                # 重新设置客户端的cookie
                self.__async_client.cookies.set('userToken', self.user_token, domain='www.4seller.com', path='/')
                # 如果future没有被终结掉手动设置终结future,否则会抛出异常影响回调执行
                if not self.__login_in_progress.done():
                    self.__login_in_progress.set_result(True)
                # 回调函数可能会失败，但是不能影响其它协程操作
                try:
                    await self.login_success_call_back(user_token)
                except Exception as e:
                    logger.error(f'FourSeller登录成功回调函数执行失败 {e}')
                    pass
            finally:
                # 提示登录任务完成
                self.__login_in_progress = None

    async def __request(self, url, method, retry_count=0, **kwargs):
        """
        4seller基本请求
        :param url: url路径
        :param method: 请求方法
        :param kwargs: 请求参数
        :return:
        """
        try:
            async with self.__sem:
                response = await self.__async_client.request(method, url, **kwargs)
        except ReadTimeout:
            raise Exception(f'FourSeller请求接口超时')
        except Exception as e:
            logger.error(f'FourSeller请求接口失败{e}')
            raise Exception(f'FourSeller请求接口失败{e}')
        else:
            try:
                data = response.json()
            except Exception as e:
                logger.error(f'FourSeller请求,json序列化失败{e},原始数据{response.text}')
                raise Exception(f'FourSeller请求,json序列化失败{e},原始数据{response.text}')
            else:
                if data.get('code') == ResponseCodeEnum.LOGIN_VALIDATION_FAILED:
                    # 刷新token
                    await self.__refresh_user_token()
                    if retry_count < 1:
                        # 最多重试一次
                        return await self.__request(url, method, 1, **kwargs)
                    else:
                        raise Exception(f'FourSeller登录成功后,再次请求登录验证失败, 请求接口{url}，返回信息{data}')
                return data

    async def __get(self, url, **kwargs):
        return await self.__request(url, 'GET', **kwargs)

    async def __post(self, url, **kwargs):
        return await self.__request(url, 'POST', **kwargs)

    async def __put(self, url, **kwargs):
        return await self.__request(url, 'PUT', **kwargs)

    async def __delete(self, url, **kwargs):
        return await self.__request(url, 'DELETE', **kwargs)

    async def __order_page_api(self, query_params: FourSellerOrderQueryModel):
        return await self.__post('/api/v2/order/page', json=query_params.model_dump(by_alias=True))

    async def __order_page_history_api(self, query_params: FourSellerOrderQueryModel):
        return await self.__post('/api/v2/order/history/page', json=query_params.model_dump(by_alias=True))

    async def order_page(self, query_params: FourSellerOrderQueryModel) -> Page[FourSellerOrderVO]:
        page_data = await self.__order_page_api(query_params)
        # 能到这里说明登录验证什么的都没有问题，接口返回代码也是代表成功
        if page_data.get('code') != ResponseCodeEnum.SUCCESS:
            raise Exception(f'FourSeller请求接口失败, 请求接口{page_data}')
        return parse_order(page_data.get('data'))

    async def order_page_history(self, query_params: FourSellerOrderQueryModel) -> Page[FourSellerOrderVO]:
        page_data = await self.__order_page_history_api(query_params)
        # 能到这里说明登录验证什么的都没有问题，接口返回代码也是代表成功
        if page_data.get('code') != ResponseCodeEnum.SUCCESS:
            raise Exception(f'FourSeller拉取历史分页订单失败, 请求接口{page_data}')
        return parse_order(page_data.get('data'))

    async def list_order(self, query_params: FourSellerOrderQueryModel) -> list[FourSellerOrderVO]:
        # 获取分页信息
        order_list = list()
        page = await self.order_page(query_params)
        # 采用并发提高速度
        page_order_task_list = list()
        for page_number in range(1, page.total_page + 1):
            page_order_task_list.append(
                self.order_page(query_params.copy(update={"page_current": page_number}))
            )
        page_list = await asyncio.gather(*page_order_task_list)
        # 所有的数据
        for every_page in page_list:
            # 插入数据
            order_list.extend(every_page.records)
        return order_list

    async def list_history_order(self, query_params: FourSellerOrderQueryModel):
        # 获取分页信息
        order_history_list = list()
        page = await self.order_page_history(query_params)
        # 采用并发提高速度
        page_order_task_list = list()
        for page_number in range(1, page.total_page + 1):
            page_order_task_list.append(self.order_page_history(
                query_params.copy(update={"page_current": page_number})
            ))
        page_list = await asyncio.gather(*page_order_task_list)
        # 所有的数据
        for every_page in page_list:
            # 插入数据
            order_history_list.extend(every_page.records)
        return order_history_list

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.__async_client.aclose()



from typing import Generic, TypeVar

from pydantic import BaseModel

T = TypeVar('T')


class Page(Generic[T], BaseModel):
    records: list[T]
    page_number: int
    page_size: int
    total: int
    total_page: int



from httpx import AsyncClient, Timeout

from ey_commerce_lib.kogan.schemas.query.order import KoganOrderQuery
from ey_commerce_lib.kogan.schemas.query.product import KoganProductQuery
from ey_commerce_lib.kogan.schemas.response.order import OrderResponseModel
from ey_commerce_lib.kogan.schemas.response.product import ProductResponse


class KoganClient:

    def __init__(self, seller_id: str, seller_token: str):
        self.seller_id = seller_id
        self.seller_token = seller_token
        timeout = Timeout(connect=5.0, read=60.0, write=10.0, pool=10.0)

        # 异步客户端
        self.__async_client = AsyncClient(base_url="https://nimda.kogan.com",
                                          headers={
                                              'Accept': 'application/json',
                                              'SellerId': self.seller_id,
                                              'SellerToken': self.seller_token
                                          },
                                          timeout=timeout
                                          )

    async def orders(self, query_params: KoganOrderQuery):
        # 设置查询参数
        orders_res = await self.__async_client.get('/api/marketplace/v2/orders/',
                                                   params=query_params.model_dump(by_alias=True,
                                                                                  exclude_none=True))
        return OrderResponseModel(**orders_res.json())

    async def products(self, query_params: KoganProductQuery):
        products_res = await self.__async_client.get('/api/marketplace/v2/products/',
                                                     params=query_params.model_dump(by_alias=True,
                                                                                    exclude_none=True
                                                                                    ))

        return ProductResponse(**products_res.json())

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.__async_client.aclose()



import json

from ey_commerce_lib.dxm.schemas.dxm_commodity_product import ViewDxmCommodityProductResponse, EditObj
from ey_commerce_lib.utils.float import truncate_decimal_str


def get_edit_commodity_product_by_view_dxm_response(data: ViewDxmCommodityProductResponse):
    """
    根据查看店小秘接口的响应
    :return:
    """
    dxm_commodity_product = data.product_dto.dxm_commodity_product
    dxm_product_customs = data.product_dto.dxm_product_customs

    product_id = dxm_commodity_product.id
    name = dxm_commodity_product.name
    name_en = dxm_commodity_product.name_en
    sku_code = dxm_commodity_product.sku_code
    sku = dxm_commodity_product.sku
    product_variation_str = ','.join(dxm_commodity_product.product_variation_str_list)
    sbm_id = dxm_commodity_product.sbm_id
    agent_id = dxm_commodity_product.agent_id
    development_id = dxm_commodity_product.development_id
    sales_id = dxm_commodity_product.sales_id
    weight = dxm_commodity_product.weight
    allow_weight_error = dxm_commodity_product.allow_weight_error
    price = dxm_commodity_product.price
    source_url = dxm_commodity_product.source_url
    img_url = dxm_commodity_product.img_url
    is_used = dxm_commodity_product.is_used
    full_cid = dxm_commodity_product.full_cid
    product_type = dxm_commodity_product.product_type
    length = dxm_commodity_product.length
    width = dxm_commodity_product.width
    height = dxm_commodity_product.height
    qc_type = dxm_commodity_product.qc_type
    product_status = dxm_commodity_product.product_status
    child_ids = dxm_commodity_product.child_ids
    child_nums = dxm_commodity_product.child_nums
    process_fee = dxm_commodity_product.process_fee
    qc_content = dxm_commodity_product.qc_content
    qc_img_num = dxm_commodity_product.qc_img_num
    group_state = dxm_commodity_product.group_state
    ncm = dxm_commodity_product.ncm
    cest = dxm_commodity_product.cest
    unit = dxm_commodity_product.unit
    origin = dxm_commodity_product.origin

    name_cn_bg = dxm_product_customs.name_cn
    name_en_bg = dxm_product_customs.name_en
    weight_bg = dxm_product_customs.weight
    price_bg = dxm_product_customs.price
    material_bg = dxm_commodity_product.material_bg
    purpose_bg = dxm_commodity_product.purpose_bg
    hgbm_bg = dxm_commodity_product.hgbm_bg
    danger_des_bg = dxm_commodity_product.danger_des_bg

    warehouse_id_list = data.product_dto.warehouse_id_list

    supplier_product_relation_map_list = data.product_dto.supplier_product_relation_map_list or []
    new_supplier_product_relation_map_list = []
    for supplier_product_relation_map in supplier_product_relation_map_list:
        supplier_id = supplier_product_relation_map.supplier_id
        is_main = supplier_product_relation_map.is_main
        new_supplier_product_relation_map_list.append({
            'supplierId': supplier_id,
            'isMain': is_main
        })

    dxm_product_packs = data.product_dto.dxm_product_packs
    # sbmId 必须为字符串
    return EditObj.model_validate({
        'dxmCommodityProduct': {
            'productId': str(product_id),
            'name': name,
            'nameEn': name_en,
            'skuCode': sku_code,
            'sku': sku,
            'productVariationStr': product_variation_str,
            'sbmId': sbm_id if sbm_id is not None else '',
            'agentId': str(agent_id),
            'developmentId': str(development_id),
            'salesId': str(sales_id),
            'weight': truncate_decimal_str(weight),
            'allowWeightError': '',
            'price': truncate_decimal_str(price),
            'sourceUrl': source_url,
            'imgUrl': img_url,
            'isUsed': is_used,
            'fullCid': full_cid,
            'productType': product_type,
            'length': length,
            'width': width,
            'height': height,
            'qcType': qc_type,
            'productStatus': 1,
            'childIds': child_ids if child_ids is not None else '',
            'childNums': child_nums if child_nums is not None else '',
            'processFee': process_fee,
            'qcContent': qc_content if qc_content is not None else '',
            'qcImgNum': qc_img_num,
            'groupState': str(group_state),
            'ncm': ncm if ncm is not None else '',
            'cest': cest if cest is not None else '',
            'unit': unit if unit is not None else '',
            'origin': origin if origin is not None else '0'
        },
        'dxmProductCustoms': {
            'nameCnBg': name_cn_bg,
            'nameEnBg': name_en_bg,
            'weightBg': truncate_decimal_str(weight_bg),
            'priceBg': truncate_decimal_str(price_bg, 4),
            'materialBg': material_bg if material_bg is not None else '',
            'purposeBg': purpose_bg if purpose_bg is not None else '',
            'hgbmBg': hgbm_bg if hgbm_bg is not None else '',
            'dangerDesBg': danger_des_bg if danger_des_bg is not None else '0'
        },
        'warehouseIdList': warehouse_id_list if warehouse_id_list is not None else '',
        'supplierProductRelationMapList': new_supplier_product_relation_map_list,
        'dxmProductPacks': dxm_product_packs
    }).model_dump_json(by_alias=True)




import json

import pytest

from ey_commerce_lib.dxm.constant.order import ORDER_SEARCH_APPROVAL_BASE_FORM, \
    ORDER_SEARCH_PENDING_PROCESSING_BASE_FORM, ORDER_SEARCH_SELF_WAREHOUSE_BASE_FORM, \
    ORDER_SEARCH_OUT_OF_STOCK_BASE_FORM, ORDER_SEARCH_HAVE_GOODS_BASE_FORM
from ey_commerce_lib.dxm.main import DxmClient
from ey_commerce_lib.dxm.schemas.warehouse import WarehouseProductQuery
from ey_commerce_lib.dxm.utils.mark import get_custom_mark_content_list_by_data_custom_mark, \
    generate_add_or_update_user_comment_data_by_content_list
from ey_commerce_lib.four_seller.main import FourSellerClient
from ey_commerce_lib.four_seller.schemas.query.order import FourSellerOrderQueryModel
from ey_commerce_lib.takesend.main import TakeSendClient
import ey_commerce_lib.dxm.utils.dxm_commodity_product as dxm_commodity_product_util


async def login_success(user_token: str):
    print(f'user_token: {user_token}')
    pass


@pytest.mark.asyncio
async def test_auto_login_4seller():
    # user_token = await auto_login_4seller(user_name="sky@eeyoung.com", password="ey010203@@")
    # print(user_token)
    async with FourSellerClient(
            user_name="xxxxx",
            password="xxxxxx",
            login_success_call_back=login_success,
            user_token="xxxxxx") as four_seller_client:
        await four_seller_client.list_history_order(FourSellerOrderQueryModel())


cookies = {
    'dxm_i': 'MTAyNzg5NiFhVDB4TURJM09EazIhZjc5ZTMxMjZmN2Q3NTk5OGZlMDg4YjQzNzNlNzlmOWY',
    'dxm_t': 'MTcxNDk2NDU4MiFkRDB4TnpFME9UWTBOVGd5IWIzZWRjYTc3YTdjYWE0MmIxMWYwNjFhOTlmMDgxZjY5',
    'dxm_c': 'TWVOMVpiclYhWXoxTlpVNHhXbUp5VmchNmM5MmM5ODc1MDBkODVjOWJmOTFlMzdhNGJiNWI3Njk',
    'dxm_w': 'ZDc3NWVhOTdkOTEyNGE4ZmRkODBmYzc2YmFlOGVjZjEhZHoxa056YzFaV0U1TjJRNU1USTBZVGhtWkdRNE1HWmpOelppWVdVNFpXTm1NUSFmZWZiZGZiMTEwMzhlN2M4M2MzYjZmYjAxYTcxMjViMw',
    'dxm_s': 'aMHO7LWKAQe6NzFrV3O4cceXkBriLGNmwondOR7INWs',
    'MYJ_MKTG_fapsc5t4tc': 'JTdCJTdE',
    'MYJ_fapsc5t4tc': 'JTdCJTIyZGV2aWNlSWQlMjIlM0ElMjJkMGRlYzNkYi00MTNlLTQyZjktOWIxNC0xYzBhZTY4MTk4ZmIlMjIlMkMlMjJ1c2VySWQlMjIlM0ElMjIxMDI3ODk2JTIyJTJDJTIycGFyZW50SWQlMjIlM0ElMjIyNTg2MzQlMjIlMkMlMjJzZXNzaW9uSWQlMjIlM0ExNzMxMDU5MDc1MjU0JTJDJTIyb3B0T3V0JTIyJTNBZmFsc2UlMkMlMjJsYXN0RXZlbnRUaW1lJTIyJTNBMTczMTA1OTA3NTQyNCUyQyUyMmxhc3RFdmVudElkJTIyJTNBMTE2JTdE',
    'MYJ_fapsc5t4tc': 'JTdCJTIyZGV2aWNlSWQlMjIlM0ElMjJkMGRlYzNkYi00MTNlLTQyZjktOWIxNC0xYzBhZTY4MTk4ZmIlMjIlMkMlMjJ1c2VySWQlMjIlM0ElMjIxMDI3ODk2JTIyJTJDJTIycGFyZW50SWQlMjIlM0ElMjIyNTg2MzQlMjIlMkMlMjJzZXNzaW9uSWQlMjIlM0ExNzMxMDU5MDc1MTEwJTJDJTIyb3B0T3V0JTIyJTNBZmFsc2UlMkMlMjJsYXN0RXZlbnRUaW1lJTIyJTNBMTczMTA1OTA3NTk5NiUyQyUyMmxhc3RFdmVudElkJTIyJTNBMTE2JTdE',
    '_dxm_ad_client_id': '7AEFA0B78CE1B35075D391C5226612CED',
    'Hm_lvt_f8001a3f3d9bf5923f780580eb550c0b': '1731459796,1731547149,1731590878,1731893161',
    'HMACCOUNT': 'F266E8F609D8D8B1',
    'MYJ_fapsc5t4tc': 'JTdCJTIyZGV2aWNlSWQlMjIlM0ElMjJkMGRlYzNkYi00MTNlLTQyZjktOWIxNC0xYzBhZTY4MTk4ZmIlMjIlMkMlMjJ1c2VySWQlMjIlM0ElMjIxMDI3ODk2JTIyJTJDJTIycGFyZW50SWQlMjIlM0ElMjIyNTg2MzQlMjIlMkMlMjJzZXNzaW9uSWQlMjIlM0ExNzMxOTAxNDMzMTk3JTJDJTIyb3B0T3V0JTIyJTNBZmFsc2UlMkMlMjJsYXN0RXZlbnRJZCUyMiUzQTExNiU3RA==',
    'Hm_lpvt_f8001a3f3d9bf5923f780580eb550c0b': '1731901433',
    'JSESSIONID': 'CD9C8372185D453012E5EEFBA64E2C9C',
}

headers = {
    'accept': 'text/html, */*; q=0.01',
    'accept-language': 'zh-CN,zh;q=0.9',
    'cache-control': 'no-cache',
    'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
    'origin': 'https://www.dianxiaomi.com',
    'pragma': 'no-cache',
    'priority': 'u=1, i',
    'referer': 'https://www.dianxiaomi.com/sys/index.htm?go=m420',
    'sec-ch-ua': '"Google Chrome";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"Windows"',
    'sec-fetch-dest': 'empty',
    'sec-fetch-mode': 'cors',
    'sec-fetch-site': 'same-origin',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36',
    'x-requested-with': 'XMLHttpRequest',
}


@pytest.mark.asyncio
async def test_dxm_api():
    async with (DxmClient(headers=headers, cookies=cookies) as dxm_client):
        # data = await dxm_client.list_order_detail_async(query=ORDER_SEARCH_HAVE_GOODS_BASE_FORM)
        # for order in data:
        #     for pair_info in order.get('detail').get('pair_info_list'):
        #         print(pair_info.get('proid'))
        data = await dxm_client.update_dxm_commodity_front_sku('17773195771232287', 'fuck112')


@pytest.mark.asyncio
async def test_warehouse():
    async with (DxmClient(headers=headers, cookies=cookies) as dxm_client):
        print(await dxm_client.page_warehouse_product(WarehouseProductQuery()))


@pytest.mark.asyncio
async def test_tasksend_api():
    async with (TakeSendClient(username="", password="") as tasksend_client):
        await tasksend_client.login()



from pydantic import BaseModel, ConfigDict

from ey_commerce_lib.utils.str import to_camel


class FourSellerOrderQueryModel(BaseModel):
    # 页码
    page_current: int = 1
    # 每页条数
    page_size: int = 100
    # 平台
    platform: str = ""
    # 店铺id列表
    shop_id_list: list[str] = []
    # 仓库id 页面未知来源
    warehouse_id: str = ""
    # 物流id 例如: 13554
    logistics_service_id: str = ""
    # 物流服务id映射关系父子id,id连接起来 例如: ["470", "13554"]
    logistics_service_map: list[str] = []
    # 单品单数: onlyOne 全部包裹类型: "" 单品多数: gtOne 多品: multiple
    item_type: str = ""
    """
    打印状态  全部: "" 面单已打印: labelPrinted   面单未打印: labelUnPrinted  装箱已打印: packingPrinted   装箱未打印: packingUnPrinted 
    拣货单已打印: pickPrinted  拣货单未打印: pickUnPrinted    
    """
    print_status: str = ""
    # 买家指定
    shipping_method: list[str] = []
    # 搜索类型  订单号: orderNo   物流跟踪号: trackNo  其余请参考页面
    search_type: str = "orderNo"
    # 搜索方式  模糊: like  精确: exact
    search_method: str = "exact"
    # 搜索内容
    search_value: str = ""
    # 商品总数最小
    item_min: str = ""
    # 商品总数最大
    item_max: str = ""
    # 重量最小
    weight_min: str = ""
    # 重量最大
    weight_max: str = ""
    # 重量单位 lb oz kg g
    weight_unit: str = "lb"
    # 到期时间 如果是数字就是小时  如果为空字符串是全部
    expires_hour: str | int = ""
    hours: str = ""
    # 平台状态 待发货: Awaiting_Shipment  待揽收: Awaiting_Collection 其它参考页面
    platform_status: str = ""
    # 发货地址id   如果为空字符串就是没有选择
    ship_from_address_id: str | int = ""
    # 国家编码 具体参考筛选页面
    country_code: str = ""
    # 国家名称 具体参考筛选页面
    country_name: str = ""
    # 省份或者洲
    state: str = ""
    # 买家备注 空字符串未选 有:1 无: 0
    buyer_note: str | int = ""
    # 卖家备注 空字符串未选 有:1 无: 0
    seller_note: str = ""
    # 系统备注: 空字符串未选 有:1 无: 0
    internal_note: str = ""
    # 平台标记 PRE_SALE: Temu预售
    platform_marker: str = ""
    # 下单时间 这个时间比上海时间要晚7个小时
    date_query_create_start_time: str = ""
    date_query_create_end_time: str = ""
    # 付款时间
    date_query_paid_start_time: str = ""
    date_query_paid_end_time: str = ""
    # 发货时间
    date_query_ship_start_time: str = ""
    date_query_ship_end_time: str = ""
    # 打印面单时间
    date_query_print_label_start_time: str = ""
    date_query_print_label_end_time: str = ""
    # 取消订单时间
    date_query_cancel_start_time: str = ""
    date_query_cancel_end_time: str = ""
    # 标签页 具体参考筛选页面
    tag_id_list: list[str] = []
    # 订单状态 全部: 空字符串  未付款: un_paid 已发货: shipped  待处理: to_ship  风控中: on_hold  处理中: in_process 已签收: delivered 参考页面
    all_order_status: str = ""
    # 订单状态: 全部: all 已发货: shipped
    order_status: str = "all"
    # 排序字段
    order_by: str = "orderPaidTime"
    # 排序方式 asc: 升序 desc: 降序
    desc: str = "asc"

    country: str = ""
    note_query: str = ""
    tag_query: str = ""
    exception_type: str = ""

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True
    )



import re
from typing import TypedDict

from lxml import html

from ey_commerce_lib.dxm.exception.common import PageInfoNotFoundException


class PageInfo(TypedDict):
    page_size: int
    total_page: int
    total_size: int
    page_number: int


def get_page_info(html_str: str) -> PageInfo:
    """
    获取分页信息和分页数据
    :return:
    """
    tree = html.fromstring(html_str)
    page_size_list = tree.xpath('//input[@id="pageSize"]/@value')
    total_page_list = tree.xpath('//input[@id="totalPage"]/@value')
    total_size_list = tree.xpath('//input[@id="totalSize"]/@value')
    page_number_list = tree.xpath('//input[@id="pageNo"]/@value')
    if len(page_size_list) == 0 or len(total_page_list) == 0:
        raise PageInfoNotFoundException("分页信息获取失败")
    return {
        'page_size': int(page_size_list[0]),
        'total_page': int(total_page_list[0]),
        'total_size': int(total_size_list[0]),
        'page_number': int(page_number_list[0])
    }


def __parse_js_vals(js_snippet: str) -> dict[str, int]:
    """
    从一段包含 jQuery .val(...) 赋值语句的 JS 代码字符串中，
    提取所有 id 和对应的数值，并返回一个 {id: int} 形式的字典。

    参数
    ----
    js_snippet : str
        包含类似 `$('#pageNo').val('1');` 的 JavaScript 代码。

    返回
    ----
    Dict[str, int]
        键是每个 .val() 前的 id（如 'pageNo'），值是对应的整数。
    """
    pattern = r"\$\('#(?P<key>\w+)'\)\.val\('(?P<value>\d+)'\);"
    matches = re.findall(pattern, js_snippet)
    return {key: int(value) for key, value in matches}


def get_purchase_pagination_info(script: str) -> dict:
    # 定义要提取的字段及其对应的 key 名
    field_map = {
        'pageNo': 'page_number',
        'pageSize': 'page_size',
        'totalSize': 'total_size',
        'totalPage': 'total_page',
    }

    result = {}

    for field, key in field_map.items():
        pattern = rf"\$\('#{field}'\)\.val\('(\d+)'\);"
        match = re.search(pattern, script)
        if not match:
            raise ValueError(f"无法从字符串中提取字段：{field}")
        result[key] = int(match.group(1))

    return result


def get_purchasing_page_info(html_str: str):
    """
    获取分页信息和分页数据(店小秘->采购部分)
    :param html_str:
    :return:
    """
    page_number = __parse_js_vals(html_str).get('pageNo')
    page_size = __parse_js_vals(html_str).get('pageSize')
    total_page = __parse_js_vals(html_str).get('totalPage')
    total_size = __parse_js_vals(html_str).get('totalSize')
    if page_number is None or page_size is None or total_page is None or total_size is None:
        raise PageInfoNotFoundException("分页信息获取失败")
    return {
        'page_size': page_size,
        'total_page': total_page,
        'total_size': total_size,
        'page_number': page_number
    }



from lxml import html

from ey_commerce_lib.dxm.schemas.warehouse import WarehouseProduct


def __get_warehouse_product_by_xpath_element(warehouse_product_element: html.HtmlElement) -> WarehouseProduct:
    proid = warehouse_product_element.xpath('.//input[@iptid="selectSingle"]/@data-proid')[0].strip()
    sku = warehouse_product_element.xpath('.//span[contains(@class, "productSku")]/text()')[0].strip()
    name_list = warehouse_product_element.xpath('.//p[contains(@class, "name")]/text()')
    name = name_list[0].strip() if len(name_list) > 0 else ''
    sku_code = warehouse_product_element.xpath('.//div[contains(@class, "skuCode")]/span/text()')[0].strip()
    img = warehouse_product_element.xpath('.//img/@src')[0].strip()
    shelf_position = warehouse_product_element.xpath('.//td[3]/text()')[0].strip()
    # 库存详情元素
    warehouse_product_stock_element = warehouse_product_element.xpath('.//td[4]//tbody/tr')[0]
    # 安全库存
    try:
        safe_stock = int(warehouse_product_stock_element.xpath('./td[1]/span/text()')[0])
    except:
        safe_stock = None
    try:
        # # 在途库存
        on_the_way_stock = int(warehouse_product_stock_element.xpath('./td[2]/span/text()')[0])
    except:
        on_the_way_stock = None
    # 未发库存
    try:
        not_shipped_stock = int(warehouse_product_stock_element.xpath('./td[3]/span/text()')[0])
    except:
        not_shipped_stock = None
    try:
        # # 占用库存
        occupy_stock = int(warehouse_product_stock_element.xpath('./td[4]/span/text()')[0])
    except:
        occupy_stock = None
    # # 可用库存
    try:
        available_stock = int(warehouse_product_stock_element.xpath('./td[5]/span/text()')[0])
    except:
        available_stock = None
    # 总量库存
    try:
        total_stock = int(warehouse_product_stock_element.xpath('./td[6]/span/text()')[0])
    except:
        total_stock = None
    # 单价
    price = float(warehouse_product_element.xpath('./td[5]/text()')[0])
    # 总价
    total_price = float(warehouse_product_element.xpath('./td[6]/text()')[0])
    # 更新时间
    update_time = warehouse_product_element.xpath('./td[7]/div[1]/text()')[0]
    # 创建时间
    create_time = warehouse_product_element.xpath('./td[7]/div[2]/text()')[0]
    return WarehouseProduct(proid=proid, sku=sku, name=name, sku_code=sku_code, img=img, shelf_position=shelf_position,
                            safe_stock=safe_stock, on_the_way_stock=on_the_way_stock,
                            not_shipped_stock=not_shipped_stock, occupy_stock=occupy_stock,
                            available_stock=available_stock, total_stock=total_stock, price=price,
                            total_price=total_price, update_time=update_time, create_time=create_time)


def list_warehouse_product(html_content: str) -> list[WarehouseProduct]:
    tree = html.fromstring(html_content)
    warehouse_product_element_list = tree.xpath('//tbody/tr[@class="content"]')
    warehouse_product_list = list()
    for warehouse_product_element in warehouse_product_element_list:
        warehouse_product_list.append(__get_warehouse_product_by_xpath_element(warehouse_product_element))
    return warehouse_product_list



from httpx import AsyncClient, Timeout

from ey_commerce_lib.takesend.config import LOGIN_HEADERS, EDIT_HEADERS


class TakeSendClient(object):

    def __init__(self, username: str, password: str):
        timeout = Timeout(connect=5.0, read=20.0, write=10.0, pool=10.0)
        self.__async_client = AsyncClient(
            base_url="http://k5.takesend.com:8180",
            timeout=timeout)
        self.__username = username
        self.__password = password

    async def login(self):
        """
        自动登录
        :return:
        """
        # 1. 先访问首页更新JESSIONID
        await self.__async_client.get("/c_index.jsp")
        # 2.在登录
        params = {
            'action': 'logon'
        }
        data = {
            'userid': self.__username,
            'password': self.__password
        }
        await self.__async_client.post("//client/Logon", params=params, data=data, headers=LOGIN_HEADERS)

    async def __client_cc_order(self, excel_data: list):
        """
        修改泰嘉产品上传重量数据
        :param excel_data:
        :return:
        """

        params = {
            'action': 'updateDweight',
        }
        data = {
            'excel[]': excel_data,
        }
        return await self.__async_client.post("/Client/CCOrder", params=params, data=data, headers=EDIT_HEADERS)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.__async_client.aclose()



LOGIN_HEADERS = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,'
              '*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'Cache-Control': 'no-cache',
    'Connection': 'keep-alive',
    'Content-Type': 'application/x-www-form-urlencoded',
    'Origin': 'http://k5.takesend.com:8180',
    'Pragma': 'no-cache',
    'Referer': 'http://k5.takesend.com:8180/c_index.jsp',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36',
}

EDIT_HEADERS = {
    'Accept': 'application/json, text/javascript, */*; q=0.01',
    'Accept-Language': 'zh-CN,zh;q=0.9',
    'Cache-Control': 'no-cache',
    'Connection': 'keep-alive',
    'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
    'Origin': 'http://k5.takesend.com:8180',
    'Pragma': 'no-cache',
    'Referer': 'http://k5.takesend.com:8180/client/Logon?action=initMenu',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36',
    'X-Requested-With': 'XMLHttpRequest',
}


from datetime import datetime
from typing import List, Optional, Dict, Any
from pydantic import BaseModel, Field, ConfigDict


class DxmProductAlbum(BaseModel):
    """商品相册信息"""
    id: int = Field(..., alias="id", description="相册ID")
    puid: int = Field(..., alias="puid", description="用户ID")
    product_id: int = Field(..., alias="productId", description="商品ID")
    name: Optional[str] = Field(None, alias="name", description="图片名称")
    img_url: Optional[str] = Field(None, alias="imgUrl", description="图片URL")
    img_original_size: int = Field(0, alias="imgOriginalSize", description="图片原始大小")
    img_compress_size: int = Field(0, alias="imgCompressSize", description="图片压缩后大小")
    img_height: int = Field(0, alias="imgHeight", description="图片高度")
    img_width: int = Field(0, alias="imgWidth", description="图片宽度")
    is_qc: int = Field(0, alias="isQc", description="是否质检图片")
    is_from_net: int = Field(1, alias="isFromNet", description="是否来自网络")
    is_del: int = Field(0, alias="isDel", description="是否删除")
    upload_state: int = Field(0, alias="uploadState", description="上传状态")
    survey_index: int = Field(0, alias="surveyIndex", description="调查索引")
    create_time: datetime = Field(..., alias="createTime", description="创建时间")
    update_time: datetime = Field(..., alias="updateTime", description="更新时间")
    is_main: Optional[int] = Field(None, alias="isMain", description="是否主图")
    id_str: Optional[str] = Field(None, alias="idStr", description="ID字符串形式")

    model_config = ConfigDict(populate_by_name=True)


class ProductVariation(BaseModel):
    """商品变体信息"""
    id: int = Field(..., alias="id", description="变体ID")
    puid: int = Field(..., alias="puid", description="用户ID")
    product_id: int = Field(..., alias="productId", description="商品ID")
    sku: str = Field(..., alias="sku", description="SKU编码")
    create_time: datetime = Field(..., alias="createTime", description="创建时间")

    model_config = ConfigDict(populate_by_name=True)


class SupplierProductRelation(BaseModel):
    """供应商商品关系"""
    supplier_name: str = Field(..., alias="supplierName", description="供应商名称")
    is_albaba: int = Field(..., alias="isAlbaba", description="是否阿里巴巴供应商")
    supplier_id: str = Field(..., alias="supplierId", description="供应商ID")
    is_main: int = Field(..., alias="isMain", description="是否主供应商")

    model_config = ConfigDict(populate_by_name=True)


class DxmWarehouseProduct(BaseModel):
    """仓库商品信息"""
    id: int = Field(..., alias="id", description="仓库商品ID")
    puid: int = Field(..., alias="puid", description="用户ID")
    warehose_id: int = Field(..., alias="warehoseId", description="仓库ID")
    goods_shelf_id: int = Field(..., alias="goodsShelfId", description="货架ID")
    shelf_name: str = Field(..., alias="shelfName", description="货架名称")
    agent_id: int = Field(0, alias="agentId", description="代理商ID")
    development_id: int = Field(..., alias="developmentId", description="开发人员ID")
    product_id: int = Field(..., alias="productId", description="商品ID")
    product_sku: str = Field(..., alias="productSku", description="商品SKU")
    full_cid: str = Field(..., alias="fullCid", description="完整分类ID")
    supplier_id: int = Field(..., alias="supplierId", description="供应商ID")
    price: float = Field(..., alias="price", description="价格")
    stock_num: int = Field(0, alias="stockNum", description="库存数量")
    safe_stock_num: int = Field(0, alias="safeStockNum", description="安全库存数量")
    presell_num: int = Field(0, alias="presellNum", description="预售数量")
    lock_stock_num: int = Field(0, alias="lockStockNum", description="锁定库存数量")
    amount: float = Field(0.0, alias="amount", description="金额")
    on_passage_num: int = Field(0, alias="onPassageNum", description="在途数量")
    on_passage_num_move: int = Field(0, alias="onPassageNumMove", description="移动中的在途数量")
    comment: str = Field("", alias="comment", description="备注")
    propasal_comment: Optional[str] = Field(None, alias="propasalComment", description="提案备注")
    comment_color: Optional[str] = Field(None, alias="commentColor", description="备注颜色")
    is_used: int = Field(1, alias="isUsed", description="是否使用")
    product_status: int = Field(0, alias="productStatus", description="商品状态")
    is_material: bool = Field(False, alias="isMaterial", description="是否原材料")
    create_time: datetime = Field(..., alias="createTime", description="创建时间")
    update_time: datetime = Field(..., alias="updateTime", description="更新时间")
    group_state: int = Field(0, alias="groupState", description="分组状态")
    allot_lock_stock_num: int = Field(0, alias="allotLockStockNum", description="分配锁定库存数量")
    warehouse_lock_stock_num: int = Field(0, alias="warehouseLockStockNum", description="仓库锁定库存数量")
    order_num: int = Field(0, alias="orderNum", description="订单数量")
    option_mode: int = Field(1, alias="optionMode", description="操作模式")
    pur_num: int = Field(0, alias="purNum", description="采购数量")
    purchasing_audit_num: int = Field(0, alias="purchasingAuditNum", description="采购审核数量")
    mark_no_purch: int = Field(1, alias="markNoPurch", description="标记不采购")
    v: int = Field(0, alias="v", description="版本号")
    pur_wait_in_num: int = Field(0, alias="purWaitInNum", description="待入库采购数量")
    available_stock_num: Optional[int] = Field(None, alias="availableStockNum", description="可用库存数量")
    oversea_lock_stock_num: int = Field(0, alias="overseaLockStockNum", description="海外锁定库存数量")
    oversea_move_pair_mark: int = Field(1, alias="overseaMovePairMark", description="海外移动配对标记")
    oversea_warehose_id: str = Field("", alias="overseaWarehoseId", description="海外仓库ID")
    full_name: Optional[str] = Field(None, alias="fullName", description="完整名称")
    sku: Optional[str] = Field(None, alias="sku", description="SKU编码")
    sku_code: Optional[str] = Field(None, alias="skuCode", description="SKU代码")
    name: Optional[str] = Field(None, alias="name", description="名称")
    img_url: Optional[str] = Field(None, alias="imgUrl", description="图片URL")
    tiny_img_url: Optional[str] = Field(None, alias="tinyImgUrl", description="缩略图URL")
    source_url: Optional[str] = Field(None, alias="sourceUrl", description="来源URL")
    sbm_id: Optional[str] = Field(None, alias="sbmId", description="SBM ID")
    sku_pro: Optional[str] = Field(None, alias="skuPro", description="SKU属性")
    name_pro: Optional[str] = Field(None, alias="namePro", description="名称属性")
    weight_pro: Optional[float] = Field(None, alias="weightPro", description="重量属性")
    price_pro: Optional[float] = Field(None, alias="pricePro", description="价格属性")
    source_url_pro: Optional[str] = Field(None, alias="sourceUrlPro", description="来源URL属性")
    comment_pro: Optional[str] = Field(None, alias="commentPro", description="备注属性")
    img_url_pro: Optional[str] = Field(None, alias="imgUrlPro", description="图片URL属性")
    is_used_pro: Optional[int] = Field(None, alias="isUsedPro", description="是否使用属性")
    full_cid_pro: Optional[str] = Field(None, alias="fullCidPro", description="完整分类ID属性")
    name_cn_bg: Optional[str] = Field(None, alias="nameCnBg", description="中文名称背景")
    name_en_bg: Optional[str] = Field(None, alias="nameEnBg", description="英文名称背景")
    weight_bg: Optional[float] = Field(None, alias="weightBg", description="重量背景")
    price_bg: Optional[float] = Field(None, alias="priceBg", description="价格背景")
    agent_name: Optional[str] = Field(None, alias="agentName", description="代理商名称")
    development_name: Optional[str] = Field(None, alias="developmentName", description="开发人员名称")
    proposal_num: Optional[int] = Field(None, alias="proposalNum", description="提案数量")
    recent_in_time: Optional[datetime] = Field(None, alias="recentInTime", description="最近入库时间")
    recent_out_time: Optional[datetime] = Field(None, alias="recentOutTime", description="最近出库时间")
    ids: Optional[List[int]] = Field(None, alias="ids", description="ID列表")
    prices: Optional[List[float]] = Field(None, alias="prices", description="价格列表")
    kc_nums: Optional[List[int]] = Field(None, alias="kcNums", description="库存数量列表")
    aqkc_nums: Optional[List[int]] = Field(None, alias="aqkcNums", description="安全库存数量列表")
    good_shelf_ids: Optional[List[int]] = Field(None, alias="goodShelfIds", description="货架ID列表")
    comments: Optional[List[str]] = Field(None, alias="comments", description="备注列表")
    full_cids: Optional[List[str]] = Field(None, alias="fullCids", description="完整分类ID列表")
    stock_num_zh: Optional[int] = Field(None, alias="stockNumZh", description="库存数量中文")
    group_num: Optional[int] = Field(None, alias="groupNum", description="分组数量")
    group_sku_table: Optional[str] = Field(None, alias="groupSkuTable", description="分组SKU表")
    old_price: Optional[float] = Field(None, alias="oldPrice", description="旧价格")
    old_stock_num: Optional[int] = Field(None, alias="oldStockNum", description="旧库存数量")
    last_pur_price: Optional[float] = Field(None, alias="lastPurPrice", description="最后采购价格")
    sale_cofficient: Optional[float] = Field(None, alias="saleCofficient", description="销售系数")
    dxm_warehose_product_g: Optional[str] = Field(None, alias="dxmWarehoseProductG", description="DXM仓库商品G")
    proposal_set: Optional[str] = Field(None, alias="proposalSet", description="提案集")
    proposal_set_json: Optional[str] = Field(None, alias="proposalSetJson", description="提案集JSON")
    this_arrival_num: Optional[int] = Field(None, alias="thisArrivalNum", description="本次到货数量")
    daily_sale: Optional[float] = Field(None, alias="dailySale", description="日销量")
    daily_sale_num: Optional[int] = Field(None, alias="dailySaleNum", description="日销售数量")
    warning_day: Optional[int] = Field(None, alias="warningDay", description="预警天数")
    purchase_plan_number: int = Field(0, alias="purchasePlanNumber", description="采购计划数量")
    warehouse_name: str = Field(..., alias="warehouseName", description="仓库名称")
    goods_shelf_lock: int = Field(0, alias="goodsShelfLock", description="货架锁定")
    state: Optional[int] = Field(None, alias="state", description="状态")
    goods_shelf_type: int = Field(0, alias="goodsShelfType", description="货架类型")
    picking_safe_stock_num: int = Field(0, alias="pickingSafeStockNum", description="拣货安全库存数量")
    oversea_product_sku: Optional[str] = Field(None, alias="overseaProductSku", description="海外商品SKU")
    authorization_id: Optional[str] = Field(None, alias="authorizationId", description="授权ID")
    sfxz: Optional[str] = Field(None, alias="sfxz", description="SFXZ")
    oversea_platfrom: Optional[str] = Field(None, alias="overseaPlatfrom", description="海外平台")
    unbilled_order_num: int = Field(0, alias="unbilledOrderNum", description="未开票订单数量")
    id_str: str = Field(..., alias="idStr", description="ID字符串形式")
    supplier_id_str: str = Field(..., alias="supplierIdStr", description="供应商ID字符串形式")
    product_id_str: str = Field(..., alias="productIdStr", description="商品ID字符串形式")
    goods_shelf_id_str: str = Field(..., alias="goodsShelfIdStr", description="货架ID字符串形式")

    model_config = ConfigDict(populate_by_name=True)


class DxmCommodityProduct(BaseModel):
    """DXM商品信息"""
    id: int = Field(..., alias="id", description="商品ID")
    puid: int = Field(..., alias="puid", description="用户ID")
    full_cid: str = Field(..., alias="fullCid", description="完整分类ID")
    name: str = Field(..., alias="name", description="商品名称")
    name_en: str = Field("", alias="nameEn", description="商品英文名称")
    spu: str = Field(..., alias="spu", description="SPU编码")
    sku: str = Field(..., alias="sku", description="SKU编码")
    sku_code: str = Field(..., alias="skuCode", description="SKU代码")
    parent_id: int = Field(..., alias="parentId", description="父商品ID")
    survey_id: int = Field(0, alias="surveyId", description="调查ID")
    data_source: Optional[str] = Field(None, alias="dataSource", description="数据来源")
    attr: Optional[str] = Field(None, alias="attr", description="属性")
    type: Optional[str] = Field(None, alias="type", description="类型")
    price: float = Field(..., alias="price", description="价格")
    process_fee: float = Field(0.0, alias="processFee", description="加工费")
    weight: float = Field(..., alias="weight", description="重量")
    allow_weight_error: float = Field(0.0, alias="allowWeightError", description="允许重量误差")
    comment: str = Field("", alias="comment", description="备注")
    propasal_comment: Optional[str] = Field(None, alias="propasalComment", description="提案备注")
    comment_color: Optional[str] = Field(None, alias="commentColor", description="备注颜色")
    img_url: str = Field(..., alias="imgUrl", description="图片URL")
    img_url_exception: int = Field(2, alias="imgUrlException", description="图片URL异常")
    source_url: str = Field(..., alias="sourceUrl", description="来源URL")
    is_used: int = Field(1, alias="isUsed", description="是否使用")
    product_status: int = Field(0, alias="productStatus", description="商品状态")
    is_del: int = Field(0, alias="isDel", description="是否删除")
    variant_or_not: int = Field(0, alias="variantOrNot", description="是否有变体")
    state: int = Field(1, alias="state", description="状态")
    is_stock: int = Field(0, alias="isStock", description="是否有库存")
    group_state: int = Field(0, alias="groupState", description="分组状态")
    sbm_id: Optional[str] = Field(None, alias="sbmId", description="SBM ID")
    agent_id: int = Field(0, alias="agentId", description="代理商ID")
    development_id: int = Field(..., alias="developmentId", description="开发人员ID")
    sales_id: int = Field(0, alias="salesId", description="销售人员ID")
    uninue_flag: int = Field(0, alias="uninueFlag", description="唯一标志")
    product_type: str = Field(..., alias="productType", description="商品类型")
    qc_type: int = Field(0, alias="qcType", description="质检类型")
    is_qc_collect: int = Field(0, alias="isQcCollect", description="是否质检收集")
    qc_template_id: int = Field(0, alias="qcTemplateId", description="质检模板ID")
    is_bind_compliance: bool = Field(False, alias="isBindCompliance", description="是否绑定合规")
    template_id: Optional[str] = Field(None, alias="templateId", description="模板ID")
    template_name: Optional[str] = Field(None, alias="templateName", description="模板名称")
    tiny_img_url: Optional[str] = Field(None, alias="tinyImgUrl", description="缩略图URL")
    length: float = Field(0.0, alias="length", description="长度")
    width: float = Field(0.0, alias="width", description="宽度")
    height: float = Field(0.0, alias="height", description="高度")
    package_length: float = Field(0.0, alias="packageLength", description="包装长度")
    package_width: float = Field(0.0, alias="packageWidth", description="包装宽度")
    package_height: float = Field(0.0, alias="packageHeight", description="包装高度")
    package_weight: float = Field(0.0, alias="packageWeight", description="包装重量")
    create_time: datetime = Field(..., alias="createTime", description="创建时间")
    update_time: datetime = Field(..., alias="updateTime", description="更新时间")
    name_cn_bg: Optional[str] = Field(None, alias="nameCnBg", description="中文名称背景")
    name_en_bg: Optional[str] = Field(None, alias="nameEnBg", description="英文名称背景")
    weight_bg: Optional[float] = Field(None, alias="weightBg", description="重量背景")
    price_bg: Optional[float] = Field(None, alias="priceBg", description="价格背景")
    material_bg: Optional[str] = Field(None, alias="materialBg", description="材料背景")
    purpose_bg: Optional[str] = Field(None, alias="purposeBg", description="用途背景")
    hgbm_bg: Optional[str] = Field(None, alias="hgbmBg", description="海关编码背景")
    danger_des_bg: Optional[str] = Field(None, alias="dangerDesBg", description="危险描述背景")
    is_customs: Optional[bool] = Field(None, alias="isCustoms", description="是否海关")
    vids_ys: Optional[str] = Field(None, alias="vidsYs", description="视频YS")
    sku_ys: Optional[str] = Field(None, alias="skuYs", description="SKU YS")
    pt_ys: Optional[str] = Field(None, alias="ptYs", description="PT YS")
    shop_id_ys: Optional[str] = Field(None, alias="shopIdYs", description="店铺ID YS")
    pid_ys: Optional[str] = Field(None, alias="pidYs", description="PID YS")
    smt_vids: Optional[str] = Field(None, alias="smtVids", description="SMT视频")
    smt_pts: Optional[str] = Field(None, alias="smtPts", description="SMT PT")
    smt_pids: Optional[str] = Field(None, alias="smtPids", description="SMT PID")
    smt_shop_ids: Optional[str] = Field(None, alias="smtShopIds", description="SMT店铺ID")
    our_vids: Optional[str] = Field(None, alias="ourVids", description="我们的视频")
    our_pts: Optional[str] = Field(None, alias="ourPts", description="我们的PT")
    our_pids: Optional[str] = Field(None, alias="ourPids", description="我们的PID")
    our_shop_ids: Optional[str] = Field(None, alias="ourShopIds", description="我们的店铺ID")
    sf_order: Optional[str] = Field(None, alias="sfOrder", description="顺丰订单")
    full_name: str = Field(..., alias="fullName", description="完整名称")
    warehose_id: Optional[str] = Field(None, alias="warehoseId", description="仓库ID")
    warehose_name: Optional[str] = Field(None, alias="warehoseName", description="仓库名称")
    invoice_save: bool = Field(False, alias="invoiceSave", description="发票保存")
    ncm: Optional[str] = Field(None, alias="ncm", description="NCM编码")
    cest: Optional[str] = Field(None, alias="cest", description="CEST编码")
    unit: Optional[str] = Field(None, alias="unit", description="单位")
    origin: Optional[str] = Field(None, alias="origin", description="产地")
    group_num: Optional[int] = Field(None, alias="groupNum", description="分组数量")
    contain_weight: float = Field(0.0, alias="containWeight", description="包含重量")
    child_ids: Optional[str] = Field(None, alias="childIds", description="子商品ID")
    child_nums: Optional[str] = Field(None, alias="childNums", description="子商品数量")
    group_sku_table: Optional[str] = Field(None, alias="groupSkuTable", description="分组SKU表")
    goods_shelf_id: Optional[str] = Field(None, alias="goodsShelfId", description="货架ID")
    good_shelf_name: Optional[str] = Field(None, alias="goodShelfName", description="货架名称")
    product_type_list: Optional[str] = Field(None, alias="productTypeList", description="商品类型列表")
    relation_flag: Optional[str] = Field(None, alias="relationFlag", description="关系标志")
    relation_str: Optional[str] = Field(None, alias="relationStr", description="关系字符串")
    ghs_count: Optional[int] = Field(None, alias="ghsCount", description="GHS计数")
    sfxz: Optional[str] = Field(None, alias="sfxz", description="SFXZ")
    arrival_num: Optional[int] = Field(None, alias="arrivalNum", description="到货数量")
    dxm_label: Optional[str] = Field(None, alias="dxmLabel", description="DXM标签")
    stock_num: Optional[int] = Field(None, alias="stockNum", description="库存数量")
    goods_shelf_name: Optional[str] = Field(None, alias="goodsShelfName", description="货架名称")
    dxm_product_customs: Optional[str] = Field(None, alias="dxmProductCustoms", description="DXM商品海关")
    dxm_commodity_product_g: Optional[str] = Field(None, alias="dxmCommodityProductG", description="DXM商品G")
    dxm_product_album_list: List[DxmProductAlbum] = Field(..., alias="dxmProductAlbumList",
                                                          description="DXM商品相册列表")
    product_variation_list: List[ProductVariation] = Field(..., alias="productVariationList",
                                                           description="商品变体列表")
    product_variation_str: Optional[str] = Field(None, alias="productVariationStr", description="商品变体字符串")
    alibaba_pair_product_list: Optional[str] = Field(None, alias="alibabaPairProductList",
                                                     description="阿里巴巴配对商品列表")
    alias_sku: Optional[str] = Field(None, alias="aliasSku", description="别名SKU")
    color: Optional[str] = Field(None, alias="color", description="颜色")
    size: Optional[str] = Field(None, alias="size", description="尺寸")
    check_detail_list: Optional[str] = Field(None, alias="checkDetailList", description="检查详情列表")
    supplier_id: Optional[str] = Field(None, alias="supplierId", description="供应商ID")
    supplier_ids: Optional[str] = Field(None, alias="supplierIds", description="供应商ID列表")
    qc_content: Optional[str] = Field(None, alias="qcContent", description="质检内容")
    qc_img_url: Optional[str] = Field(None, alias="qcImgUrl", description="质检图片URL")
    qc_img_num: int = Field(0, alias="qcImgNum", description="质检图片数量")
    product_packs: Optional[str] = Field(None, alias="productPacks", description="商品包装")
    main_supplier_name: Optional[str] = Field(None, alias="mainSupplierName", description="主供应商名称")
    main_supplier_id: Optional[str] = Field(None, alias="mainSupplierId", description="主供应商ID")
    dxm_supplier_list_str: Optional[str] = Field(None, alias="dxmSupplierListStr", description="DXM供应商列表字符串")
    supplier_size: int = Field(0, alias="supplierSize", description="供应商大小")
    variant_sku_count: int = Field(0, alias="variantSkuCount", description="变体SKU计数")
    current_sku_count: int = Field(0, alias="currentSkuCount", description="当前SKU计数")
    reference_by_group: int = Field(0, alias="referenceByGroup", description="按组引用")
    agent_name: Optional[str] = Field(None, alias="agentName", description="代理商名称")
    development_name: Optional[str] = Field(..., alias="developmentName", description="开发人员名称")
    sales_name: Optional[str] = Field(None, alias="salesName", description="销售人员名称")
    productuu_id: Optional[str] = Field(None, alias="productuuId", description="商品UUID")
    auth_id: Optional[str] = Field(None, alias="authId", description="授权ID")
    ov_warehose_id: Optional[str] = Field(None, alias="ovWarehoseId", description="OV仓库ID")
    product_amount_price: Optional[str] = Field(None, alias="productAmountPrice", description="商品金额价格")
    is_pair_oversea: int = Field(0, alias="isPairOversea", description="是否配对海外")
    id_str: str = Field(..., alias="idStr", description="ID字符串形式")
    product_variation_str_list: List[str] = Field(..., alias="productVariationStrList",
                                                  description="商品变体字符串列表")
    qc_template_id_str: str = Field(..., alias="qcTemplateIdStr", description="质检模板ID字符串")
    goods_shelf_id_str: Optional[str] = Field(None, alias="goodsShelfIdStr", description="货架ID字符串形式")

    model_config = ConfigDict(populate_by_name=True)


class DxmProductCustoms(BaseModel):
    """DXM商品海关信息"""
    id: int = Field(..., alias="id", description="海关信息ID")
    product_id: int = Field(..., alias="productId", description="商品ID")
    name_cn: str = Field(..., alias="nameCn", description="中文名称")
    name_en: str = Field(..., alias="nameEn", description="英文名称")
    price: float = Field(..., alias="price", description="价格")
    weight: float = Field(..., alias="weight", description="重量")
    material: str = Field("", alias="material", description="材料")
    purpose: str = Field("", alias="purpose", description="用途")
    hgbm: str = Field(..., alias="hgbm", description="海关编码")
    danger_des: int = Field(0, alias="dangerDes", description="危险描述")
    puid: int = Field(..., alias="puid", description="用户ID")
    create_time: datetime = Field(..., alias="createTime", description="创建时间")
    update_time: datetime = Field(..., alias="updateTime", description="更新时间")
    sku: Optional[str] = Field(None, alias="sku", description="SKU编码")
    id_str: str = Field(..., alias="idStr", description="ID字符串形式")

    model_config = ConfigDict(populate_by_name=True)


class ProductDTO(BaseModel):
    """商品数据传输对象"""
    dxm_commodity_product: DxmCommodityProduct = Field(..., alias="dxmCommodityProduct", description="DXM商品信息")
    dxm_product_customs: DxmProductCustoms = Field(..., alias="dxmProductCustoms", description="DXM商品海关信息")
    dxm_warehose_list: Optional[List[Any]] = Field(None, alias="dxmWarehoseList", description="DXM仓库列表")
    warehouse_id_list: Optional[List[Any]] = Field(None, alias="warehouseIdList", description="仓库ID列表")
    dxm_quality_check_template: Optional[Any] = Field(None, alias="dxmQualityCheckTemplate", description="DXM质检模板")
    dxm_supplier_product_relation_list: Optional[List[Any]] = Field(None, alias="dxmSupplierProductRelationList",
                                                                    description="DXM供应商商品关系列表")
    dxm_commodity_product_list: Optional[List[Any]] = Field(None, alias="dxmCommodityProductList",
                                                            description="DXM商品列表")
    spu: Optional[str] = Field(None, alias="spu", description="SPU编码")
    dxm_product_packs: List[Any] = Field(..., alias="dxmProductPacks", description="DXM商品包装列表")
    supplier_product_relation_map_list: Optional[List[SupplierProductRelation]] = Field(...,
                                                                              alias="supplierProductRelationMapList",
                                                                              description="供应商商品关系映射列表")
    dxm_warehouse_product_list: Optional[List[DxmWarehouseProduct]] = Field(..., alias="dxmWarehouseProductList",
                                                                  description="DXM仓库商品列表")
    count: Optional[int] = Field(0, alias="count", description="计数")

    model_config = ConfigDict(populate_by_name=True)


class ViewDxmCommodityProductResponse(BaseModel):
    """查看DXM商品信息响应"""
    product_dto: ProductDTO = Field(..., alias="productDTO", description="商品数据传输对象")

    model_config = ConfigDict(populate_by_name=True)


# =============================编辑部分=========================================

class EditDxmCommodityProduct(BaseModel):
    product_id: str = Field("", alias="productId")
    name: str
    name_en: str = Field("", alias="nameEn")
    sku_code: str = Field("", alias="skuCode")
    sku: str
    product_variation_str: str = Field("", alias="productVariationStr")
    sbm_id: str = Field("", alias="sbmId")
    agent_id: str = Field("", alias="agentId")
    development_id: str = Field("", alias="developmentId")
    sales_id: str = Field("", alias="salesId")
    weight: str
    allow_weight_error: str = Field("", alias="allowWeightError")
    price: str
    source_url: str = Field("", alias="sourceUrl")
    img_url: str = Field("", alias="imgUrl")
    is_used: int = Field(..., alias="isUsed")
    full_cid: str = Field(..., alias="fullCid")
    product_type: str = Field(..., alias="productType")
    length: int
    width: int
    height: int
    qc_type: int = Field(..., alias="qcType")
    product_status: int = Field(..., alias="productStatus")
    child_ids: str = Field("", alias="childIds")
    child_nums: str = Field("", alias="childNums")
    process_fee: int = Field(..., alias="processFee")
    qc_content: str = Field("", alias="qcContent")
    qc_img_str: str = Field("", alias="qcImgStr")
    qc_img_num: int = Field("", alias="qcImgNum")
    group_state: str = Field("", alias="groupState")
    ncm: str = Field("", alias="ncm")
    cest: str = Field("", alias="cest")
    unit: str = Field("", alias="unit")
    origin: str = Field("", alias="origin")


class EditDxmProductCustoms(BaseModel):
    name_cn_bg: str = Field(..., alias="nameCnBg")
    name_en_bg: str = Field(..., alias="nameEnBg")
    weight_bg: str = Field(..., alias="weightBg")
    price_bg: str = Field(..., alias="priceBg")
    material_bg: str = Field("", alias="materialBg")
    purpose_bg: str = Field("", alias="purposeBg")
    hgbm_bg: str = Field("", alias="hgbmBg")
    danger_des_bg: str = Field(..., alias="dangerDesBg")


class EditSupplierProductRelationMap(BaseModel):
    supplier_id: str = Field(..., alias="supplierId")
    is_main: int = Field(..., alias="isMain")


class EditObj(BaseModel):
    dxm_commodity_product: EditDxmCommodityProduct = Field(..., alias="dxmCommodityProduct")
    dxm_product_customs: EditDxmProductCustoms = Field(..., alias="dxmProductCustoms")
    warehouse_id_list: str = Field("", alias="warehouseIdList")
    supplier_product_relation_map_list: List[EditSupplierProductRelationMap] = Field(..., alias="supplierProductRelationMapList")
    dxm_product_packs: List = Field(..., alias="dxmProductPacks")

